<?php
declare(strict_types=1);

namespace app\controllers;

use Yii;
use yii\filters\AccessControl;
use yii\web\Controller;
use yii\web\Response;
use yii\web\Cookie;
use yii\web\UploadedFile;

use app\models\Supply;
use app\models\SupplyGroup;

class ProfileController extends Controller
{
    /**
     * {@inheritdoc}
     */
    public function behaviors()
    {
        return [
            'access' => [
                'class' => AccessControl::class,
                'rules' => [
                    [
                        'allow' => true,
                        'roles' => ['@']
                    ],
                    [
                        'allow' => false,
                        'roles' => ['?'],
                        'verbs' => ['POST'],
                        'denyCallback' => [$this, 'accessDenied']
                    ]
                ]
            ]
        ];
    }

    public function accessDenied()
    {
        $cookies = Yii::$app->response->cookies;

        $cookies->add(new Cookie([
            'name' => 'redirect',
            'value' => Yii::$app->request->pathInfo
        ]));

        Yii::$app->response->format = Response::FORMAT_JSON;

        // Проверить переадресацию на уровне сервера
        Yii::$app->response->content = json_encode([
            'main' => $this->renderPartial('/account'),
            'redirect' => '/authentication',
            '_csrf' => Yii::$app->request->getCsrfToken()
        ]);
    }

    /**
     * Страница с настройками аккаунта
     */
    public function actionIndex(): string|array
    {
        // Инициализация
        $model = Yii::$app->user->identity;

        // Генерация
        $sidebar = $this->renderPartial('sidebar');

        if (Yii::$app->request->isAjax) {
            // AJAX-POST-запрос

            Yii::$app->response->format = Response::FORMAT_JSON;

            return [
                'main' => $this->renderPartial('index', compact('model', 'sidebar')),
                'redirect' => '/profile',
                '_csrf' => Yii::$app->request->getCsrfToken()
            ];
        }

        return $this->render('index', compact('model', 'sidebar'));
    }

    /**
     * Страницка поставок
     */
    public function actionSupplies(): string|array
    {
        // Инициализация
        $model = new Supply(Yii::$app->request->post('Supply') ?? Yii::$app->request->get('Supply'));

        // Генерация
        $sidebar = $this->renderPartial('sidebar', compact('model'));

        if (Yii::$app->request->isAjax) {
            // AJAX-POST-запрос

            Yii::$app->response->format = Response::FORMAT_JSON;

            return [
                'main' => $this->renderPartial('supplies', compact('model', 'sidebar')),
                'redirect' => '/profile/supplies',
                '_csrf' => Yii::$app->request->getCsrfToken()
            ];
        }

        return $this->render('supplies', compact('model', 'sidebar'));
    }

    public function actionImport()
    {
        // Инициализация
        $model = new Supply(Yii::$app->request->post('Supply') ?? Yii::$app->request->get('Supply'));
        $model->scenario = $model::SCENARIO_IMPORT;

        // Генерация
        $sidebar = $this->renderPartial('sidebar', compact('model'));

        if (Yii::$app->request->isAjax) {
            // AJAX-POST-запрос

            Yii::$app->response->format = Response::FORMAT_JSON;

            $model->file = UploadedFile::getInstances($model, 'file');

            if (!$test = $model->import()) {
                Yii::$app->response->statusCode = 409;
            }

            return [
                'main' => $this->renderPartial('supplies', compact('model', 'sidebar')),
                '_csrf' => Yii::$app->request->getCsrfToken()
            ];
        }

        return $this->render('supplies', compact('model', 'sidebar'));
    }

    public static function readGroups()
    {
        // Инициализация
        $groups = [];

        foreach (SupplyGroup::readAll() as $group) {
            // Перебор всех групп

            // Генерация [КЛЮЧ => ИМЯ]
            $groups[$group->_key] = $group->name;
        }

        return $groups;
    }
}
