<?php
declare(strict_types=1);

namespace app\models;

use Yii;

use mirzaev\yii2\arangodb\ActiveRecord;

use Exception;

/**
 * Документ
 */
abstract class Document extends ActiveRecord
{
    /**
     * Имя коллекции
     */
    public static function collectionName(): string
    {
        return throw new Exception('Не установлено название коллекции');
    }

    /**
     * Свойства
     */
    public function attributes(): array
    {
        return [
            '_key',
            'date',
            'writer'
        ];
    }

    /**
     * Метки свойств
     */
    public function attributeLabels(): array
    {
        return [
            'date' => 'Дата',
            'writer' => 'Аккаунт записавшего'
        ];
    }


    /**
     * Правила
     *
     * @todo Добавить проверку существования аккаунта
     */
    public function rules(): array
    {
        return [
            [
                'writer',
                'string'
            ]
        ];
    }

    /**
     * Перед сохранением
     *
     * @todo Подождать обновление от ебаного Yii2 и добавить
     * проверку типов передаваемых параметров
     */
    public function beforeSave($data): bool
    {
        if (parent::beforeSave($data)) {
            if ($this->isNewRecord) {
            }

            $this->date = time();
            $this->writer = $this->writer ?? Yii::$app->user->id;

            return true;
        }

        return false;
    }

    /**
     * Чтение идентификатора
     */
    public function readId(): ?string
    {
        return isset($this->_key) && static::collectionName() ? static::collectionName() . '/' . $this->_key : null;
    }

    /**
     * Поиск по идентификатору
     */
    public static function searchById(string $_id): ?static
    {
        return static::findOne(['_id' => $_id]);
    }

    /**
     * Чтение количества записей
     */
    public static function readAmount(): int
    {
        return static::find()->count();
    }
}
