<?php
declare(strict_types=1);

namespace app\models;

/**
 * Ребро
 */
abstract class Edge extends Document
{
    /**
     * Свойства
     */
    public function attributes(): array
    {
        return array_merge(
            parent::attributes(),
            [
                '_from',
                '_to',
                'type'
            ]
        );
    }

    /**
     * Метки свойств
     */
    public function attributeLabels(): array
    {
        return array_merge(
            parent::attributeLabels(),
            [
                'date' => 'От кого',
                'date' => 'К кому',
                'type' => 'Тип'
            ]
        );
    }

    /**
     * Правила
     */
    public function rules(): array
    {
        return array_merge(
            parent::rules(),
            [
                [
                    ['_from', '_to'],
                    'required',
                    'message' => 'Заполните поле: {attribute}'
                ],
                [
                    'type',
                    'string'
                ]
            ]
        );
    }

    /**
     * Записать
     */
    public static function write(string $_from, string $_to, string $type = '', array $data = []): ?static
    {
        // Инициализация
        $edge = new static;

        // Настройка
        $edge->_from = $_from;
        $edge->_to = $_to;
        $edge->type = $type;

        foreach ($data as $key => $value) {
            if(is_int($key)) {
                // Если ключ задан автоматически

                $edge->{$value} = true;
            } else {
                // Иначе ключ записан вручную

                $edge->{$key} = $value;
            }
        }

        // Запись
        $edge->save();

        return $edge;
    }


    /**
     * Перед сохранением
     *
     * @todo Подождать обновление от ебаного Yii2 и добавить
     * проверку типов передаваемых параметров
     */
    public function beforeSave($data): bool
    {
        if (parent::beforeSave($data)) {
            if ($this->isNewRecord) {
            }

            $this->type = $this->type ?? '';

            return true;
        }

        return false;
    }
}
